package xmlunleashed.ch10;

import javax.servlet.*;
import javax.servlet.http.*;

import java.io.*;
import java.util.*;

import javax.xml.bind.*;
import javax.xml.marshal.*;

import xmlunleashed.ch10.jaxb.*;

/**
 *  This servlet processes XML documents for rental properties. <br>
 *
 *  The servlet uses the RentalPropertyDAO to retrieve information for
 *  rental properties.  The objects returned are then marshalled as an
 *  XML document using the JAXB framework.
 */
public class RentalXMLServlet extends HttpServlet {

	/**
	 *  The rental property data access object (DAO)
	 */
	protected RentalPropertyDAO myRentalDAO;

	/**
	 *  Create an instance of the RentalPropertyDAO
	 */
	public void init() throws ServletException {

		// retrieve database connection parameters
		String dbUrl = getInitParameter("dbUrl");
		String driverName = getInitParameter("driverName");
		String user = getInitParameter("user");
		String pass = getInitParameter("pass");

		// create an instance of the RentalPropertyDAO
		try {
			myRentalDAO = new RentalPropertyDAO(driverName, dbUrl, user, pass);
		}
		catch (Exception exc) {
			log(exc.toString());
			throw new ServletException(exc);
		}
	}

	/**
	 *  Perform the following steps for GET requests.
	 *
	 *  <ol>
	 *  <li>Retrieve a list of rental properties from RentalPropertyDAO<li>
	 *  <li>Marshal the list as an XML document<li>
	 *  </ol>
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException {

		ServletOutputStream out = null;
		RentalPropertyList theList = null;

		try {
			// Set the content type to text/xml
			response.setContentType("text/xml");

			// Retrieve the servlet output stream
			out = response.getOutputStream();

			// Retrieve a list of rental properties
			theList = myRentalDAO.getRentalProperties();

			// Marshal the list as an XML document
			theList.marshal(out);
		}
		catch (DAOException exc) {
			exc.getRootCause().printStackTrace();
			throw new ServletException(exc.getRootCause());
		}
		catch (Exception exc) {
			log(exc.toString());
			exc.printStackTrace();
			throw new ServletException(exc);
		}
		finally {
			out.close();
		}
	}
}
