
package xmlunleashed.ch10.jaxb;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import javax.xml.bind.ConversionException;
import javax.xml.bind.Dispatcher;
import javax.xml.bind.Element;
import javax.xml.bind.InvalidAttributeException;
import javax.xml.bind.LocalValidationException;
import javax.xml.bind.MarshallableObject;
import javax.xml.bind.Marshaller;
import javax.xml.bind.MissingContentException;
import javax.xml.bind.NoValueException;
import javax.xml.bind.StructureValidationException;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.Validator;
import javax.xml.marshal.XMLScanner;
import javax.xml.marshal.XMLWriter;
import xmlunleashed.ch10.jaxb.Address;
import xmlunleashed.ch10.jaxb.Contact;


public class RentalProperty
    extends MarshallableObject
    implements Element
{

    private String _PropId;
    private String _Name;
    private Address _Address;
    private double _SquareFootage;
    private boolean has_SquareFootage = false;
    private double _Bedrooms;
    private boolean has_Bedrooms = false;
    private double _Bath;
    private boolean has_Bath = false;
    private BigDecimal _Price;
    private Contact _Contact;

    public String getPropId() {
        return _PropId;
    }

    public void setPropId(String _PropId) {
        this._PropId = _PropId;
        if (_PropId == null) {
            invalidate();
        }
    }

    public String getName() {
        return _Name;
    }

    public void setName(String _Name) {
        this._Name = _Name;
        if (_Name == null) {
            invalidate();
        }
    }

    public Address getAddress() {
        return _Address;
    }

    public void setAddress(Address _Address) {
        this._Address = _Address;
        if (_Address == null) {
            invalidate();
        }
    }

    public double getSquareFootage() {
        if (has_SquareFootage) {
            return _SquareFootage;
        }
        throw new NoValueException("square_footage");
    }

    public void setSquareFootage(double _SquareFootage) {
        this._SquareFootage = _SquareFootage;
        has_SquareFootage = true;
        invalidate();
    }

    public boolean hasSquareFootage() {
        return has_SquareFootage;
    }

    public void deleteSquareFootage() {
        has_SquareFootage = false;
        invalidate();
    }

    public double getBedrooms() {
        if (has_Bedrooms) {
            return _Bedrooms;
        }
        throw new NoValueException("bedrooms");
    }

    public void setBedrooms(double _Bedrooms) {
        this._Bedrooms = _Bedrooms;
        has_Bedrooms = true;
        invalidate();
    }

    public boolean hasBedrooms() {
        return has_Bedrooms;
    }

    public void deleteBedrooms() {
        has_Bedrooms = false;
        invalidate();
    }

    public double getBath() {
        if (has_Bath) {
            return _Bath;
        }
        throw new NoValueException("bath");
    }

    public void setBath(double _Bath) {
        this._Bath = _Bath;
        has_Bath = true;
        invalidate();
    }

    public boolean hasBath() {
        return has_Bath;
    }

    public void deleteBath() {
        has_Bath = false;
        invalidate();
    }

    public BigDecimal getPrice() {
        return _Price;
    }

    public void setPrice(BigDecimal _Price) {
        this._Price = _Price;
        if (_Price == null) {
            invalidate();
        }
    }

    public Contact getContact() {
        return _Contact;
    }

    public void setContact(Contact _Contact) {
        this._Contact = _Contact;
        if (_Contact == null) {
            invalidate();
        }
    }

    public void validateThis()
        throws LocalValidationException
    {
        if (_PropId == null) {
            throw new MissingContentException("prop_id");
        }
        if (_Name == null) {
            throw new MissingContentException("name");
        }
        if (_Address == null) {
            throw new MissingContentException("address");
        }
        if (!has_SquareFootage) {
            throw new MissingContentException("square_footage");
        }
        if (!has_Bedrooms) {
            throw new MissingContentException("bedrooms");
        }
        if (!has_Bath) {
            throw new MissingContentException("bath");
        }
        if (_Price == null) {
            throw new MissingContentException("price");
        }
        if (_Contact == null) {
            throw new MissingContentException("contact");
        }
    }

    public void validate(Validator v)
        throws StructureValidationException
    {
        v.validate(_Address);
        v.validate(_Contact);
    }

    public void marshal(Marshaller m)
        throws IOException
    {
        XMLWriter w = m.writer();
        w.start("rental_property");
        w.leaf("prop_id", _PropId.toString());
        w.leaf("name", _Name.toString());
        m.marshal(_Address);
        if (has_SquareFootage) {
            w.leaf("square_footage", Double.toString(_SquareFootage));
        }
        if (has_Bedrooms) {
            w.leaf("bedrooms", Double.toString(_Bedrooms));
        }
        if (has_Bath) {
            w.leaf("bath", Double.toString(_Bath));
        }
        w.leaf("price", _Price.toString());
        m.marshal(_Contact);
        w.end("rental_property");
    }

    public void unmarshal(Unmarshaller u)
        throws UnmarshalException
    {
        XMLScanner xs = u.scanner();
        Validator v = u.validator();
        xs.takeStart("rental_property");
        while (xs.atAttribute()) {
            String an = xs.takeAttributeName();
            throw new InvalidAttributeException(an);
        }
        if (xs.atStart("prop_id")) {
            xs.takeStart("prop_id");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _PropId = String.valueOf(s);
            } catch (Exception x) {
                throw new ConversionException("prop_id", x);
            }
            xs.takeEnd("prop_id");
        }
        if (xs.atStart("name")) {
            xs.takeStart("name");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _Name = String.valueOf(s);
            } catch (Exception x) {
                throw new ConversionException("name", x);
            }
            xs.takeEnd("name");
        }
        _Address = ((Address) u.unmarshal());
        {
            xs.takeStart("square_footage");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _SquareFootage = Double.parseDouble(s);
            } catch (Exception x) {
                throw new ConversionException("square_footage", x);
            }
            has_SquareFootage = true;
            xs.takeEnd("square_footage");
        }
        {
            xs.takeStart("bedrooms");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _Bedrooms = Double.parseDouble(s);
            } catch (Exception x) {
                throw new ConversionException("bedrooms", x);
            }
            has_Bedrooms = true;
            xs.takeEnd("bedrooms");
        }
        {
            xs.takeStart("bath");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _Bath = Double.parseDouble(s);
            } catch (Exception x) {
                throw new ConversionException("bath", x);
            }
            has_Bath = true;
            xs.takeEnd("bath");
        }
        if (xs.atStart("price")) {
            xs.takeStart("price");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _Price = new BigDecimal(s);
            } catch (Exception x) {
                throw new ConversionException("price", x);
            }
            xs.takeEnd("price");
        }
        _Contact = ((Contact) u.unmarshal());
        xs.takeEnd("rental_property");
    }

    public static RentalProperty unmarshal(InputStream in)
        throws UnmarshalException
    {
        return unmarshal(XMLScanner.open(in));
    }

    public static RentalProperty unmarshal(XMLScanner xs)
        throws UnmarshalException
    {
        return unmarshal(xs, newDispatcher());
    }

    public static RentalProperty unmarshal(XMLScanner xs, Dispatcher d)
        throws UnmarshalException
    {
        return ((RentalProperty) d.unmarshal(xs, (RentalProperty.class)));
    }

    public boolean equals(Object ob) {
        if (this == ob) {
            return true;
        }
        if (!(ob instanceof RentalProperty)) {
            return false;
        }
        RentalProperty tob = ((RentalProperty) ob);
        if (_PropId!= null) {
            if (tob._PropId == null) {
                return false;
            }
            if (!_PropId.equals(tob._PropId)) {
                return false;
            }
        } else {
            if (tob._PropId!= null) {
                return false;
            }
        }
        if (_Name!= null) {
            if (tob._Name == null) {
                return false;
            }
            if (!_Name.equals(tob._Name)) {
                return false;
            }
        } else {
            if (tob._Name!= null) {
                return false;
            }
        }
        if (_Address!= null) {
            if (tob._Address == null) {
                return false;
            }
            if (!_Address.equals(tob._Address)) {
                return false;
            }
        } else {
            if (tob._Address!= null) {
                return false;
            }
        }
        if (has_SquareFootage) {
            if (!tob.has_SquareFootage) {
                return false;
            }
            if (_SquareFootage!= tob._SquareFootage) {
                return false;
            }
        } else {
            if (tob.has_SquareFootage) {
                return false;
            }
        }
        if (has_Bedrooms) {
            if (!tob.has_Bedrooms) {
                return false;
            }
            if (_Bedrooms!= tob._Bedrooms) {
                return false;
            }
        } else {
            if (tob.has_Bedrooms) {
                return false;
            }
        }
        if (has_Bath) {
            if (!tob.has_Bath) {
                return false;
            }
            if (_Bath!= tob._Bath) {
                return false;
            }
        } else {
            if (tob.has_Bath) {
                return false;
            }
        }
        if (_Price!= null) {
            if (tob._Price == null) {
                return false;
            }
            if (!_Price.equals(tob._Price)) {
                return false;
            }
        } else {
            if (tob._Price!= null) {
                return false;
            }
        }
        if (_Contact!= null) {
            if (tob._Contact == null) {
                return false;
            }
            if (!_Contact.equals(tob._Contact)) {
                return false;
            }
        } else {
            if (tob._Contact!= null) {
                return false;
            }
        }
        return true;
    }

    public int hashCode() {
        int h = 0;
        h = ((127 *h)+((_PropId!= null)?_PropId.hashCode(): 0));
        h = ((127 *h)+((_Name!= null)?_Name.hashCode(): 0));
        h = ((127 *h)+((_Address!= null)?_Address.hashCode(): 0));
        h = ((31 *h)+((int)(Double.doubleToLongBits(_SquareFootage)^(Double.doubleToLongBits(_SquareFootage)>>> 32))));
        h = ((31 *h)+((int)(Double.doubleToLongBits(_Bedrooms)^(Double.doubleToLongBits(_Bedrooms)>>> 32))));
        h = ((31 *h)+((int)(Double.doubleToLongBits(_Bath)^(Double.doubleToLongBits(_Bath)>>> 32))));
        h = ((127 *h)+((_Price!= null)?_Price.hashCode(): 0));
        h = ((127 *h)+((_Contact!= null)?_Contact.hashCode(): 0));
        return h;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer("<<rental_property");
        if (_PropId!= null) {
            sb.append(" prop_id=");
            sb.append(_PropId.toString());
        }
        if (_Name!= null) {
            sb.append(" name=");
            sb.append(_Name.toString());
        }
        if (_Address!= null) {
            sb.append(" address=");
            sb.append(_Address.toString());
        }
        if (has_SquareFootage) {
            sb.append(" square_footage=");
            sb.append(Double.toString(_SquareFootage));
        }
        if (has_Bedrooms) {
            sb.append(" bedrooms=");
            sb.append(Double.toString(_Bedrooms));
        }
        if (has_Bath) {
            sb.append(" bath=");
            sb.append(Double.toString(_Bath));
        }
        if (_Price!= null) {
            sb.append(" price=");
            sb.append(_Price.toString());
        }
        if (_Contact!= null) {
            sb.append(" contact=");
            sb.append(_Contact.toString());
        }
        sb.append(">>");
        return sb.toString();
    }

    public static Dispatcher newDispatcher() {
        return Address.newDispatcher();
    }

}
